@@ -1,14 +1,18 @@
-# $Id: Wheel.pm,v 1.13 2001/04/30 19:46:53 rcaputo Exp $
+# $Id: Wheel.pm,v 1.17 2002/11/05 21:03:32 rcaputo Exp $
 
 package POE::Wheel;
 
 use strict;
-use Carp;
+
+use vars qw($VERSION);
+$VERSION = (qw($Revision: 1.17 $ ))[1];
+
+use Carp qw(croak);
 
 # Used to generate unique IDs for wheels.  This is static data, shared
 # by all.
 my $next_id = 1;
-my %active_socketfactory_ids;
+my %active_wheel_ids;
 
 sub new {
   my $type = shift;
@@ -17,13 +21,14 @@ sub new {
 
 sub allocate_wheel_id {
   while (1) {
-    last unless exists $active_socketfactory_ids{ $next_id++ };
+    last unless exists $active_wheel_ids{ ++$next_id };
   }
-  return $active_socketfactory_ids{$next_id} = $next_id;
+  return $active_wheel_ids{$next_id} = $next_id;
 }
 
 sub free_wheel_id {
-  delete $active_socketfactory_ids{shift};
+  my $id = shift;
+  delete $active_wheel_ids{$id};
 }
 
 #------------------------------------------------------------------------------
@@ -87,7 +92,7 @@ occur.
 
   $wheel->event( InputEvent   => 'new_input_event',
                  ErrorEvent   => undef,
-                 FlushedEvenc => 'new_flushed_event',
+                 FlushedEvent => 'new_flushed_event',
                );
 
 =back
@@ -110,7 +115,7 @@ flushed to its filehandle.
 
 =head1 STATIC FUNCTIONS
 
-These functions keep global information about all weels.  They should
+These functions keep global information about all wheels.  They should
 be called as normal functions:
 
   &POE::Wheel::function( ... );
@@ -119,8 +124,8 @@ be called as normal functions:
 
 =item allocate_wheel_id
 
-allocate_wheel_id() allocates a uniquely identifier for a wheel.
-Wheels pass these identifiers back to sessions in their events so that
+allocate_wheel_id() allocates a unique identifier for a wheel.  Wheels
+pass these identifiers back to sessions in their events so that
 sessions with several wheels can match events back to other
 information.
 
